@@ -4,28 +4,38 @@ use strict;
 use warnings;
 use Exporter qw/import/;
 
-our $VERSION   = q{0.96};
-our @EXPORT    = qw(dispatch on);
-our @EXPORT_OK = qw(dispatch on);
+our $VERSION       = q{0.97};
+our @EXPORT        = qw(dispatch on cases);
+our @EXPORT_OK     = qw(dispatch on cases);
+
+my $DISPATCH_TABLE = {};
+
+sub cases () {
+    return keys %$DISPATCH_TABLE;
+}
 
 sub dispatch (&@) {
     my $code_ref  = shift;    # catch sub ref that was coerced from the 'dispatch' BLOCK
     my $match_ref = shift;    # catch the input reference passed after the 'dispatch' BLOCK
 
     # build up dispatch table for each k/v pair preceded by 'on'
-    my $dispatch = {};
     while ( my $key = shift @_ ) {
         my $HV = shift @_;
-        $dispatch->{$key} = _to_sub($HV);
+        $DISPATCH_TABLE->{$key} = _to_sub($HV);
     }
 
     # call $code_ref that needs to return a valid bucket name
     my $key = $code_ref->($match_ref);
 
-    die qq{Computed static bucket not found\n} if not $dispatch->{$key} or 'CODE' ne ref $dispatch->{$key};
+    die qq{Computed static bucket not found\n} if not $DISPATCH_TABLE->{$key} or 'CODE' ne ref $DISPATCH_TABLE->{$key};
 
-    # call subroutine ref defined as the v in the k/v $dispatch->{$key} slot
-    return $dispatch->{$key}->();
+    # call subroutine ref defined as the v in the k/v $DISPATCH_TABLE->{$key} slot
+    my $sub_to_call = $DISPATCH_TABLE->{$key};
+
+    # reset table
+    $DISPATCH_TABLE = {};
+
+    $sub_to_call->();
 }
 
 # on accumulater, wants h => v pair, where h is a static bucket string and v is a sub ref
@@ -51,15 +61,15 @@ Dispatch::Fu - Converts any complicated conditional dispatch situation into fami
   use strict;
   use warnings;
   use Dispatch::Fu qw/dispatch on/; # exported by default, just for show here
-  
-  my $CASES = [qw/1 2 3 4 5/];
-  
+
+  my $INPUT = [qw/1 2 3 4 5/];
+
   my $results = dispatch {                       # <~ start of 'dispatch' construct
-      my $cases_ref = shift;                     # <~ input reference
-      return ( scalar @$cases_ref > 5 )          # <~ return a string that must be
+      my $input_ref = shift;                     # <~ input reference
+      return ( scalar @$input_ref > 5 )          # <~ return a string that must be
        ? q{case5}                                #    defined below using the 'on'
-       : sprintf qq{case%d}, scalar @$cases_ref; #    keyword, this i
-  } $CASES,                                      # <~ input reference, SCALAR passed to dispatch BLOCK 
+       : sprintf qq{case%d}, scalar @$input_ref; #    keyword, this i
+  } $INPUT,                                      # <~ input reference, SCALAR passed to dispatch BLOCK
     on case0 => sub { print qq{case 0\n}; 0},    # <~ if dispatch returns 'case0', run this CODE
     on case1 => sub { print qq{case 1\n}; 1} ,   # <~ if dispatch returns 'case1', run this CODE
     on case2 => sub { print qq{case 2\n}; 2},    #    ...   ...   ...   ...   ...   ...   ...
@@ -78,7 +88,7 @@ This module presents a generic structure that can be used to implement all of th
 attemts to bring things to Perl like, I<switch> or I<case> statements, I<given>/I<when>,
 I<smartmatch>, etc.
 
-=head2 The Problem 
+=head2 The Problem
 
 C<HASH> based dispatching in Perl is a very fast and well established way
 to organize your code.  A dispatch table can be fashioned easily when the
@@ -90,18 +100,18 @@ that is fundamentally based on a 1:1 mapping of a value of C<$action> to a
 C<HASH> key defined in C<$dispatch>:
 
   my $CASE = get_case(); # presumed to return one of the hash keys used below
-   
+
   my $dispatch = {
     do_dis  => sub { ... },
     do_dat  => sub { ... },
     do_deez => sub { ... },
     do_doze => sub { ... },
-  }; 
-   
+  };
+
   if (not $CASE or not exists $dispatch->{$CASE}) {
     die qq{case not supported\n};
   }
-  
+
   my $results = $dispatch->{$CASE}->();
 
 But this nice situation breaks down if C<$CASE> is a value that is not suitable
@@ -110,7 +120,7 @@ C<$CASE>) is not sufficient to determine what case to dispatch. C<Dispatch::Fu>
 solves this problem by providing a stage where a static key might be computed
 or classified.
 
-=head2 The Solution 
+=head2 The Solution
 
 C<Dispatch::Fu> solves the problem by providing a I<Perlish> and I<idiomatic>
 hook for computing a static key from an arbitrarily defined algorithm written
@@ -132,7 +142,7 @@ as follows:
    on do_dat  => sub { ... },
    on do_deez => sub { ... },
    on do_doze => sub { ... };
-  
+
 The one difference here is, if C<$case> is defined but not accounted for
 using the C<on> keyword, then C<dispatch> will throw an exception via
 C<die>. Certainly any logic meant to deal with the value (or lack thereof)
@@ -142,8 +152,8 @@ An example of a more complicated scenario for generating the static key might
 be defined, follows:
 
   my $results = dispatch {
-    my $cases = shift;
-    my $rand  = $cases->[0];
+    my $input_ref = shift;
+    my $rand  = $input_ref->[0];
     if ( $rand < 2.5 ) {
         return q{do_dis};
     }
@@ -184,34 +194,57 @@ or point to anything a Perl scalar reference can point to. It's the single
 point of entry for input.
 
   my $results = dispatch {
-  
-    my $cases_ref = shift; # <~ there is only one parameter, but can a reference to anything
+    my $input_ref = shift; # <~ there is only one parameter, but can a reference to anything
     my $key = q{default};  # <~ initiate the default key to use, 'default' by convention not required
     ...                    # <~ compute $key (yada yada)
     return $key;           # <~ key must be limited to the set of keys added with C<on>
-  },
+  }
   ...
 
 The C<dispatch> implementation must return a static string, and that string
 should be one of the keys added using the C<on> keyword. Otherwise, an exception
 will be thrown via C<die>.
 
+=item C<cases>
+
+This routine is for introspection inside of the C<dispatch> BLOCK. It returns
+the list of all cases added by the C<on> routine. Outside of the C<dispatch>
+BLOCK, it returns an empty C<HASH> reference.
+
+B<Note:> do not rely on the ordering of these cases to be consistent; it relies
+on the C<keys> keyword, which operates on C<HASH>es and key order is therefore
+not deterministic.
+
+Given the full example above,
+
+  my $results = dispatch {
+    my $input_ref = shift;
+    ...
+    my @cases = cases; # (qw/do_dis do_dat do_deez do_doze/)
+    ...
+  },
+  [ rand 10 ],
+   on do_dis  => sub { ... },
+   on do_dat  => sub { ... },
+   on do_deez => sub { ... },
+   on do_doze => sub { ... };
+
 =item C<REF>
 
 This is the singular scalar reference that contains all the stuff to be used
 in the C<dispatch> BLOCK. In the example above it is, C<[rand 10]>. It is
 the way to pass arbitrary data into C<dispatch>. E.g.,
 
-  my $CASES  = [qw/foo bar baz 1 3 4 5/];
-  
+  my $INPUT  = [qw/foo bar baz 1 3 4 5/];
+
   my $result = dispatch {
 
-    my $cases_ref = shift; # <~ there is only one parameter, but can a reference to anything
+    my $input_ref = shift; # <~ there is only one parameter, but can a reference to anything
     my $key = q{default};  # <~ initiate the default key to use, 'default' by convention not required
     ...                    # <~ compute $key (yada yada)
     return $key;           # <~ key must be limited to the set of keys added with C<on>
 
-  } $CASES,                ### <><~ the single scalar reference to be passed to the C<dispatch> BLOCK
+  } $INPUT,                ### <><~ the single scalar reference to be passed to the C<dispatch> BLOCK
   ...
 
 =item C<on>
@@ -220,16 +253,16 @@ This keyword builds up the dispatch table. It consists of a static string and
 a subroutine reference. In order for this to work for you, the C<dispatch>
 BLOCK must return strictly only the keys that are defined via C<on>.
 
-  my $CASES = [qw/foo bar baz 1 3 4 5/];
-    
+  my $INPUT = [qw/foo bar baz 1 3 4 5/];
+
   my $results = dispatch {
-  
-    my $cases_ref = shift; # <~ there is only one parameter, but can a reference to anything
+
+    my $input_ref = shift; # <~ there is only one parameter, but can a reference to anything
     my $key = q{default};  # <~ initiate the default key to use, 'default' by convention not required
     ...                    # <~ compute $key (yada yada)
     return $key;           # <~ key must be limited to the set of keys added with C<on>
-  
-  } $CASES,                ### <><~ the single scalar reference to be passed to the C<dispatch> BLOCK
+
+  } $INPUT,                ### <><~ the single scalar reference to be passed to the C<dispatch> BLOCK
    on case1 => sub { ... },
    on case2 => sub { ... },
    on case3 => sub { ... },
@@ -244,19 +277,19 @@ one to manage what happens in each C<on> case more explicitly. Perl's scoping
 rules lets one use variables in a higher scope, so that would be the way to
 deal with it. E.g.,
 
-  my $CASES = [qw/foo bar baz 1 3 4 5/];
-   
+  my $INPUT = [qw/foo bar baz 1 3 4 5/];
+
   my $results = dispatch {
-  
-    my $cases_ref = shift;       # there is only one parameter, but can a reference to anything
-    my $key        = q{default}; # initiate the default key to use, 'default' by convention not required
-    ...                          # compute $key
-    return $key;                 # key must be limited to the set of keys added with C<on>
-  
-  } $CASES,                      # <~ the single scalar reference to be passed to the C<dispatch> BLOCK
-   on q{default}  => sub { return do_default($CASES)                      },
-   on q{key1}     => sub { return do_key1(cases => $CASES)                },
-   on q{key2}     => sub { return do_key2(qw/some other inputs entirely/) };
+
+    my $input_ref = shift;      # there is only one parameter, but can a reference to anything
+    my $key       = q{default}; # initiate the default key to use, 'default' by convention not required
+    ...                         # compute $key
+    return $key;                # key must be limited to the set of keys added with C<on>
+
+  } $INPUT,                     # <~ the single scalar reference to be passed to the C<dispatch> BLOCK
+   on q{default}  => sub { do_default($INPUT)                      },
+   on q{key1}     => sub { do_key1(cases => $INPUT)                },
+   on q{key2}     => sub { do_key2(qw/some other inputs entirely/) };
 
 =back
 
